/*
 * gsampler-instrument.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <glib/gi18n-lib.h>

#include "gsampler-instrument.h"
#include "gsampler-window.h"
#include "gsampler-utils.h"
#include "gsampler-client.h"
#include "gsampler-client-channel.h"

G_DEFINE_TYPE (GsamplerInstrument, gsampler_instrument, G_TYPE_OBJECT);

static void 
gsampler_instrument_init (GsamplerInstrument * self) {
	self->priv = GSAMPLER_INSTRUMENT_GET_PRIVATE (self);
	self->priv->file = NULL;
	self->priv->inst_list = NULL;
}

static void 
gsampler_instrument_finalize (GObject* obj) {
	GsamplerInstrument* self;
	self = (GsamplerInstrument*) obj;
	g_free (self->priv->file);
	g_id_list_free (self->priv->inst_list, TRUE);
	G_OBJECT_CLASS (gsampler_instrument_parent_class)->finalize (obj);
}

static void 
gsampler_instrument_class_init (GsamplerInstrumentClass * klass) {
	gsampler_instrument_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerInstrumentPrivate));
	G_OBJECT_CLASS (klass)->finalize = gsampler_instrument_finalize;
}

GsamplerInstrument* 
gsampler_instrument_new (void) {
	GsamplerInstrument* self = NULL;

	self = (GsamplerInstrument*) g_object_new (GSAMPLER_TYPE_INSTRUMENT, NULL);
	return self;
}

gboolean
gsampler_instrument_load_file (GsamplerInstrument* self, const gchar* file_name) {
	GsamplerClient* client;
	gchar** inst_list = NULL;
	gint element_it;
	gint inst_list_lenght = 0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (file_name != NULL, FALSE);

	if (!g_file_test (file_name, G_FILE_TEST_IS_REGULAR)) {
		gsampler_main_window_send_message ("warning", _("Can't read file"));
		return FALSE;
	}

	g_id_list_free (self->priv->inst_list, TRUE);
	self->priv->inst_list = NULL;
	g_free (self->priv->file);
#ifdef G_OS_WIN32
	self->priv->file = string_replace (file_name,"\\", "/");
#else
	self->priv->file = g_strdup (file_name);
#endif
	
	client = gsampler_client_get_instance ();
	inst_list = gsampler_client_get_file_instrument_list (client, self->priv->file);
	if (g_strcmp0 (inst_list[0], "No file") == 0) {
		gsampler_main_window_send_message ("warning", _("Can't get instrument list"));
		g_strfreev (inst_list);
		g_object_unref (client);
		return FALSE;
	}

	if (inst_list != NULL)
		inst_list_lenght = g_strv_length (inst_list);
	for (element_it = 0; element_it < inst_list_lenght; element_it ++) {
		gchar* element;
		gint inst_id;
		InstrumentInfo *inst_info;
		
		element = g_strdup (inst_list[element_it]);
		inst_id = atoi (element);
		inst_info = gsampler_client_get_file_instrument_info (client, self->priv->file, inst_id);
		if (inst_info->name != NULL) {
			self->priv->inst_list = g_id_list_add (self->priv->inst_list, inst_id, g_strdup (inst_info->name));
		}
		g_free (element);
	}
	g_strfreev (inst_list);
	g_object_unref (client);
	return TRUE;
}

gchar*
gsampler_instrument_get_file (GsamplerInstrument* self) {
	g_return_val_if_fail (self != NULL, NULL);
	
	return g_strdup (self->priv->file);
}

gchar* 
gsampler_instrument_get_name (GsamplerInstrument* self, gint id) {
	gchar* result = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	
	result = (gchar*) g_id_list_get (self->priv->inst_list, id);
	return g_strdup (result);
}

GSList* 
gsampler_instrument_get_list (GsamplerInstrument* self) {
	g_return_val_if_fail (self != NULL, NULL);

	return self->priv->inst_list;
}